############
# File I/O #
############

proc :w {{fname ""} {overwrite no} {create no}} {
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  # get all overlays
  if {[llength [::overlay::ids]] <=0} {
    return -code break {Nothing to save}
  }
  if {$fname == {}} {
    if {[set fname [tk_getSaveFile \
      -initialdir $initdir \
      -filetypes { \
        {{IMD files} {.imd}} \
        {{Text files} {.txt}} \
        {{All files} *} \
        } \
      -defaultextension {.imd}  \
      ]] == {}} {
      return -code break {Save cancelled}
    }
  }
  setup configure System,LastDir [file dirname $fname]
  
  if {[set fp [open $fname w]] eq {}} {
    return -code error "Couldn't open file $fname"
  }
  # disable storage of recorded signal
  set propList [::overlay::getProperty]
  foreach prop {Record} {
    set i [lsearch $propList $prop]
    set propList [lreplace $propList $i $i]
  }

  # use export to do this
  # write each overlay to a file and fill index file
  foreach id [::overlay::ids] {
    set p [list]
    puts $fp "$id {"
    foreach prop $propList {
      lappend p $prop [getProp $id $prop]
    }
    puts $fp $p
    puts $fp "}"
  }
  close $fp

  # success
  return -code ok "Saved to $fname"
}

proc :l {{fname {}} {replace no} {ignore no}} {
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  if {$fname == {}} {
    if {[set fname [tk_getOpenFile \
      -initialdir $initdir \
      -filetypes { \
        {{IMD files} {.imd}} \
        {{Text files} {.txt}} \
        {{All files} *} \
        } \
      -defaultextension {.imd}  \
      ]] == {}} {
      return -code break {Load cancelled}
    }
  }
  setup configure System,LastDir [file dirname $fname]
  if {[set fp [open $fname r]] eq {}} {
    return -code error "Couldn't open file $fname"
  }

  # read file at once
  set def [read $fp]
  close $fp
  
  # remove comments
  set def [regsub -all {^#.*} $def { }]
  # remove newlines
  set def [regsub -all {\n} $def { }]
  # remove "bad" symbols
  set def [regsub -all {[\[\]]} $def { }]

  # delete old overlays if wanted
  if {$replace} {
    ::overlay::delete all
    ::TkEsweepXYPlot::removeTrace IMD all
  }
  # parse
  foreach {ID propList} $def {
    set id [::overlay::add $propList]
    # force display update
    ::overlay::modify $id [list FR [getProp $id FR] Show [getProp $id Show]]
  }

  # success
  return -code ok "$fname loaded" 
}

proc :export {ID {fname {}}} {
  set id [isID $ID]
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  if {$fname == {}} {
    if {[set fname [tk_getSaveFile \
      -initialdir $initdir \
      -filetypes { \
        {{DAT files} {.dat}} \
        {{Text files} {.txt}} \
        {{All files} *} \
        } \
      -defaultextension {.dat}  \
      ]] == {}} {
      return -code break {Save cancelled}
    }
  }
  setup configure System,LastDir [file dirname $fname]
  
  if {[file extension $fname] ne {.txt}} {
    # binary save
    # add meta data for an easy import
    foreach prop {Name Color SigDef OutLvl RMS} {
      lappend meta $prop [getProp $ID $prop]
    }
    esweep::save -filename $fname -obj [getProp $id Record] -meta $meta
  } else {
    # ascii save
    esweep::toAscii -filename $fname -obj [getProp $id Record]
  }
  return -code ok "Exported to $fname"
}

proc :import {{fname {}}} {
  if {[setup cget System,LastDir] eq {}} {
    set initdir [file join [pwd]]
  } else {
    set initdir [setup cget System,LastDir]
  }
  if {$fname == {}} {
    if {[set fname [tk_getOpenFile \
      -initialdir $initdir \
      -filetypes { \
        {{IMD files} {.dat}} \
        {{All files} *} \
        } \
      -defaultextension {.dat}  \
      ]] == {}} {
      return -code break {Load cancelled}
    }
  }
  setup configure System,LastDir [file dirname $fname]

  # only binary data allowed
  set import [esweep::load -filename $fname -meta props]
  lappend props Record $import

  set id [::overlay::add $props]
  return #$id
}

