# $from and $to are in dBA, n is the number of steps, vlimit is the maximum allowed output voltage
macro level {type from to n vlimit} {
  switch -glob -- $type {
    tw* {
      :set Signal,Definition,Bandwidth {3000 20000}
    }
    mid* {
      :set Signal,Definition,Bandwidth {300 3000}
    }
    w* {
      :set Signal,Definition,Bandwidth {20 300}
    }
    lo* {
      :set Signal,Definition,Bandwidth {20 3000}
    }
    default {
      :set Signal,Definition,Bandwidth {20 20000}
    }
  }
  :set Signal,Definition,Bins 10/2
  :set Signal,Definition,Spacing log

  :set Measure,Mode dual

  set colors [list red purple darkblue darkgreen brown black yellow] 
  
  set dbstep [expr {double($to-$from)/($n-1)}]
  set step [expr {pow(10, $dbstep/20.0)}]

  # set initial output level
  :set Output,Level 0.01
  set ID [:measure default]
  :eval $ID -weighting A
  set sigdef [:query $ID SigDef]

  # adjust the output level to reach the desired first SPL step
  set k [expr {pow(10, ($from-[:query $ID RMSweighted])/20.0)}]
  :set Output,Level [expr {$k*[:set Output,Level]}]

  # check if we will have reached the limit
  if {[expr {$k*[:query $ID OutLvl]}] >= $vlimit} {
    :message {Maximum allowed output voltage reached}
    return -code continue
  }

  for {set i 0} {$i < $n} {incr i} {
    if {[catch {set ID [:measure default]} msg]} {
      :message $msg
      break
    }
    # normalize
    :expr $ID / [expr {pow(10, ($i*$dbstep+$from)/20.0)}]
    :copy $ID
    set ID [:paste]
    :eval $ID -weighting A
    set rms [:query $ID RMSweighted]
    set td [expr {100*pow(10, ([:query $ID TD]-[:query $ID RMSweighted])/20)}]

    :modify $ID Name [format "%.1f dBA - %.1f%% TD @ %.1f Vrms" [expr {$rms+$i*$dbstep+$from}] $td [:query $ID OutLvl]] Color [lindex $colors [expr {$i % [llength $colors]}]]

    :set Output,Level [expr {$step*[:set Output,Level]}]
    if {[expr {$step*[:query $ID OutLvl]}] >= $vlimit} {
      :message {Maximum allowed output voltage reached}
      break
    }
  }
  :modify [:master] Show no
  return -code continue
}
