# Copyright 2016 Jochen Fabricius
# Distribution and modification is encouraged

package provide macro 1.1

namespace eval ::macro {
  variable macros
  variable bodies
  variable aliases [list]
  variable win {}
}

proc ::macro::addAlias {args} {
  variable aliases
  foreach cmd $args {
    lappend aliases $cmd
  }
}

proc ::macro::useTk {winID} {
  variable win
  set win $winID
}

proc ::macro::import {file} {
  variable macros
  variable bodies
  # create a super safe interpreter, which knows only two commands:
  # "source" and "macro"
  # The "macro" command is just an alias for "proc"
  set ip [interp create -safe]
  # hide every command except these two above
  foreach cmd [$ip eval info commands] {
    $ip hide $cmd
  }
  $ip expose source
  $ip expose proc macro

  set file [join $file]
  # now source the file
  if {[catch {$ip eval source -encoding [encoding system] $file} msg]} {
    return -code error $msg
  }

  $ip hide source
  $ip hide macro
  $ip expose info

  # insert the new macros in the array
  foreach cmd [$ip eval info commands] {
    if {[catch {$ip eval info body $cmd}]} {
      continue
    }
    lappend macros $cmd
    set bodies($cmd) [$ip eval info body $cmd]
    set bodies($cmd,args) [$ip eval info args $cmd]
    if {[llength $bodies($cmd,args)] == 0} {
      set bodies($cmd,args) [list {args}]
    }
  }
}

proc ::macro::exec {name args} {
  variable macros
  variable bodies
  variable aliases
  variable win

  if {[lsearch $macros $name] < 0} {
    return -code error "Unknown macro: $name"
  }

  # create new interpreter
  set ip [::safe::interpCreate]
  # add aliases
  foreach alias $aliases {
    $ip alias $alias $alias
  }
  interp share {} stdout $ip
  interp share {} stdin $ip
  interp share {} stderr $ip
  if {$win ne {}} {
    ::safe::loadTk $ip -use $win
  }
  ::safe::interpConfigure $ip -accessPath [file join [pwd] macros]
  # execute macro
  set lambda [list [list $bodies($name,args) $bodies($name)]]
  set code [catch {$ip eval apply $lambda {*}$args} result]

  # destroy interpreter
  interp delete $ip

  return -code $code $result
}

#### test
#proc alias_test {} {
#  puts stderr {Testing aliases}
#}
#::macro::addAlias alias_test
#::macro::import test.mcr
#::macro::exec test 1 2
